package com.cf.cs.authentication.misc;


import com.cf.cs.authentication.configure.CasbinConfigure;
import com.cf.cs.authentication.pojo.po.AuthAccountUser;
import com.cf.cs.authentication.service.AuthUsersService;
import com.cf.cs.database.mapper.RoleMapper;
import com.cf.cs.database.pojo.entity.Role;
import com.cf.cs.database.service.RoleResourceService;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.casbin.jcasbin.main.Enforcer;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import static com.cf.cs.database.pojo.entity.table.RoleTableDef.ROLE;

/**
 * The type Casbin authorize utils.
 *
 * @author xajhuang @163.com
 */
@Component
@Slf4j
public class CasbinAuthorizeUtils {
    private static final ConcurrentHashMap<String, Enforcer> ENFORCER_POOL = new ConcurrentHashMap<>();

    @Resource
    private CasbinConfigure casbinConfigure;

    @Resource
    private AuthUsersService authUsersService;

    @Resource
    private RoleResourceService roleResourceService;

    @Resource
    private RoleMapper roleMapper;

    /**
     * Init.
     */
    @PostConstruct
    public void init() {
        roleMapper.selectAll().forEach(role -> {
            if (!CasbinAuthorizeUtils.ENFORCER_POOL.containsKey(role.getName())) {
                String baseDir = System.getProperty("user.dir");
                String model   = baseDir + "/" + casbinConfigure.getModelPath();
                String policy  = baseDir + "/" + casbinConfigure.getPolicyPath();

                Enforcer e = new Enforcer(model, policy);
                e.enableAutoSave(false);

                List<Role> subRole = roleMapper.selectListByCondition(ROLE.ID.ge(role.getId()));
                subRole.forEach(k -> {
                    e.addRoleForUser(role.getName(), k.getName());
                    log.trace("+++{} has {}", role.getName(), k.getName());
                });

                // 添加角色资源权限
                roleResourceService.getRoleResourceByRoleId(role.getId()).forEach(k -> {
                    e.addPermissionForUser(role.getName(), k.getResources().getPath(), k.getResources().getHttpMethod(),
                                           Boolean.TRUE.equals(k.getAuthorize()) ? "allow" : "deny");
                    log.trace("++++++[{}, {}, {}: {}]", role.getName(), k.getResources().getPath(), k.getResources().getHttpMethod(),
                              Boolean.TRUE.equals(k.getAuthorize()) ? "allow" : "deny");
                });

                // 缓存当前权限验证器
                CasbinAuthorizeUtils.ENFORCER_POOL.put(role.getName(), e);
            }
        });
    }

    /**
     * Casbin verify authorize boolean.
     *
     * @param userName   the username
     * @param url        the url
     * @param httpMethod the http method
     * @return the boolean
     */
    public boolean casbinVerifyAuthorize(String userName, String url, String httpMethod) {
        AuthAccountUser user = authUsersService.getAuthAccountByUserName(userName);

        if (user == null) {
            return false;
        }

        // 角色被禁用
        if (Boolean.FALSE.equals(user.getRole().getAvailable())) {
            return false;
        }

        Enforcer e = CasbinAuthorizeUtils.ENFORCER_POOL.get(user.getRole().getName());

        if (e != null) {
            // 支持权限继承
            for (String s : e.getImplicitRolesForUser(user.getRole().getName())) {
                Enforcer se = CasbinAuthorizeUtils.ENFORCER_POOL.get(s);

                if (s != null && (se.enforce(s, url, httpMethod))) {
                    return true;
                }
            }
        }
        return false;
    }
}
