package com.cf.cs.authentication.security.impl;

import com.cf.cs.authentication.exception.CommonAuthException;
import com.cf.cs.authentication.interceptor.RequestBodyCacheWrapper;
import com.cf.cs.authentication.misc.JwtUtils;
import com.cf.cs.authentication.misc.SecurityResponseUtils;
import com.cf.cs.authentication.service.AccountJwtCacheService;
import com.cf.cs.base.common.ConstValue;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.misc.ApiContextUtils;
import com.cf.cs.base.pojo.po.JwtCache;
import com.cf.cs.database.pojo.entity.User;
import com.cf.cs.database.service.OperationLogDataBaseService;
import io.micrometer.common.util.StringUtils;
import jakarta.annotation.Resource;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;

/**
 * The type Jwt authentication filter.
 *
 * @author xajhuang @163.com
 */
@Component
@Slf4j
public class JwtAuthenticationFilter extends OncePerRequestFilter {
    @Resource
    private JwtUtils jwtUtils;

    @Resource
    private UserDetailServiceImpl userDetailsService;

    @Resource
    private AccountJwtCacheService accountJwtCacheService;

    @Resource
    private OperationLogDataBaseService optLogDbService;

    @Override
    protected void doFilterInternal(HttpServletRequest request,
                                    @NotNull HttpServletResponse response,
                                    @NotNull FilterChain filterChain) throws ServletException, IOException {
        String jwt  = request.getHeader(jwtUtils.getHeadValue());

        HttpServletRequest requestWrapper = new RequestBodyCacheWrapper(request);

        // 这里如果没有jwt，继续往后走，因为后面还有鉴权管理器等去判断是否拥有身份凭证，所以是可以放行的
        // 没有jwt相当于匿名访问，若有一些接口是需要权限的，则不能访问这些接口
        if (StringUtils.isBlank(jwt) || StringUtils.isEmpty(jwt)) {
            filterChain.doFilter(requestWrapper, response);
            return;
        }

        try {
            User     user     = jwtUtils.parseToken(jwt);
            JwtCache jwtCache = accountJwtCacheService.getAccountJwtCache(user.getUid());

            if (jwtCache == null) {
                handleException(requestWrapper, response, ErrorCode.ERR_TOKENNOTFOUND);
                return;
            }

            // 缓存 JWT 以及用户信息
            ApiContextUtils.setJwtUserInfo(jwtCache.getJwt(), user.getUsername(), user.getUid(), user.getId());

            // 刷新缓存
            if (System.currentTimeMillis() - jwtCache.getAddTimestamp() >= ConstValue.SECOND_OF_MINUTE * ConstValue.MS_OF_SECONDS) {
                accountJwtCacheService.deleteAccountJwtCache(jwtCache.getUId());
                accountJwtCacheService.createAccountJwtCache(jwtCache.getUId(), jwtCache.getJwt());
            }

            UsernamePasswordAuthenticationToken token = new UsernamePasswordAuthenticationToken(user.getUsername(), null,
                                                                                                userDetailsService.getUserAuthority(
                                                                                                        user.getUid()));
            SecurityContextHolder.getContext().setAuthentication(token);
            // 放行
            filterChain.doFilter(requestWrapper, response);
        } catch (CommonAuthException e) {
            handleException(requestWrapper, response, e.getErr());
        } finally {
            ApiContextUtils.clear();
        }
    }

    private void handleException(HttpServletRequest request, HttpServletResponse response, ErrorCode err) throws IOException {
        // 特定处理逻辑，例如设置响应状态、记录日志、重定向等
        response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
        SecurityResponseUtils.authenticationResponse(request, response, err, err.getDescription(), optLogDbService);
    }
}
