package com.cf.cs.base.misc;


import com.cf.cs.base.common.ConstValue;
import com.cf.cs.base.config.ObjectMapperProvider;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.ConstraintViolation;
import jakarta.validation.Validation;
import jakarta.validation.ValidatorFactory;
import org.hibernate.validator.HibernateValidator;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.charset.StandardCharsets;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * The type Helper utils.
 *
 * @author xajhuang @163.com
 */
public class HelperUtils {
    private HelperUtils() {
        throw new AssertionError("Instantiating utility class.");
    }

    private static final ObjectMapper OBJ_MAPPER = ObjectMapperProvider.getMapper();

    /**
     * Gets json.
     *
     * @param <T> the type parameter
     * @param obj the obj
     * @return the json
     * @throws JsonProcessingException the json processing exception
     */
    public static <T> String getJson(T obj) throws JsonProcessingException {
        return OBJ_MAPPER.writeValueAsString(obj);
    }

    /**
     * Truncate string string.
     *
     * @param orgString the org string
     * @param maxLength the max length
     * @return the string
     */
    public static String truncateString(String orgString, int maxLength) {
        if (!stringNotEmptyOrNull(orgString) || orgString.length() <= maxLength) {
            return orgString;
        } else {
            return orgString.substring(0, maxLength - 4) + "...";
        }
    }

    /**
     * Bytes to hex string.
     *
     * @param bArray the b array
     * @return the string
     */
    public static String bytesToHexString(byte[] bArray) {
        StringBuilder sb = new StringBuilder(bArray.length);
        String        sTemp;
        for (byte b : bArray) {
            sTemp = Integer.toHexString(0xFF & b);
            if (sTemp.length() < 2) {
                sb.append(0);
            }
            sb.append(sTemp.toUpperCase());
        }
        return sb.toString();
    }

    /**
     * Convert string to gb 2312 byte [ ].
     *
     * @param str the str
     * @return the byte [ ]
     * @throws UnsupportedEncodingException the unsupported encoding exception
     */
    public static byte[] convertStringToGb2312(String str) throws UnsupportedEncodingException {
        String utf8    = new String(str.getBytes(StandardCharsets.UTF_8));
        String unicode = new String(utf8.getBytes(), StandardCharsets.UTF_8);
        String gbk     = new String(unicode.getBytes("GB2312"));
        return gbk.getBytes();
    }

    /**
     * Input stream 2 string.
     *
     * @param inputStream the input stream
     * @return the string
     * @throws IOException the io exception
     */
    public static String inputStream2String(InputStream inputStream) throws IOException {
        if (inputStream != null) {
            final int           bufferSize = 1024;
            final char[]        buffer     = new char[bufferSize];
            final StringBuilder out        = new StringBuilder();
            Reader              in         = new InputStreamReader(inputStream, StandardCharsets.UTF_8);

            int charsRead;
            while ((charsRead = in.read(buffer, 0, buffer.length)) > 0) {
                out.append(buffer, 0, charsRead);
            }

            return out.toString();
        } else {
            return "";
        }
    }

    /**
     * Gets timestamp milli second.
     *
     * @param dateTime the date time
     * @return the timestamp milli second
     */
    public static long getTimestampMilliSecond(String dateTime) {
        return Timestamp.valueOf(dateTime).toInstant().toEpochMilli();
    }

    /**
     * Gets current datetime.
     *
     * @return the current datetime
     */
    public static String getCurrentDatetime() {
        return LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
    }

    /**
     * Validate list.
     *
     * @param <T>    the type parameter
     * @param t      the t
     * @param groups the groups
     * @return the list
     */
    public static <T> List<String> validate(T t, Class<?>... groups) {
        Set<ConstraintViolation<T>> set;
        ValidatorFactory validatorFactory = Validation.byProvider(HibernateValidator.class)
                                                      .configure()
                                                      .failFast(false)
                                                      .buildValidatorFactory();

        if (groups != null && groups.length > 0) {
            set = validatorFactory.getValidator().validate(t, groups);
        } else {
            set = validatorFactory.getValidator().validate(t);
        }
        List<String> validateError = new ArrayList<>();
        if (!set.isEmpty()) {
            int index = 0;
            for (ConstraintViolation<T> val : set) {
                validateError.add("[" + index++ + "]: " + val.getMessage());
            }
        }
        return validateError;
    }

    /**
     * String not empty or null boolean.
     *
     * @param str the str
     * @return the boolean
     */
    public static boolean stringNotEmptyOrNull(String str) {
        return str != null && !str.isEmpty();
    }

    /**
     * Meagre db string value string.
     *
     * @param str the str
     * @return the string
     */
    public static String meagreDbStringValue(String str) {
        return stringNotEmptyOrNull(str) ? str : null;
    }

    /**
     * Gets http request headers.
     *
     * @param request the request
     * @return the http request headers
     * @throws JsonProcessingException the json processing exception
     */
    public static String getHttpRequestHeaders(HttpServletRequest request) throws JsonProcessingException {
        if (request != null) {
            Map<String, String> headerMap   = new HashMap<>(15);
            Enumeration<String> enumeration = request.getHeaderNames();
            while (enumeration.hasMoreElements()) {
                String name  = enumeration.nextElement();
                String value = request.getHeader(name);
                headerMap.put(name, value);
            }
            return getJson(headerMap);
        } else {
            return "{}";
        }
    }

    /**
     * Gets http request src ip.
     *
     * @param request the request
     * @return the http request src ip
     * @throws UnknownHostException the unknown host exception
     */
    @SuppressWarnings({"java:S3776", "java:S2692", "java:S1066"})
    public static String getHttpRequestSrcIp(HttpServletRequest request) throws UnknownHostException {
        String unknown     = "unknown";
        String localhostIp = "127.0.0.1";
        // 客户端与服务器同为一台机器，获取的 ip 有时候是 ipv6 格式
        String localhostIpv6 = "0:0:0:0:0:0:0:1";
        String separator     = ",";

        if (request == null) {
            return "unknown";
        }
        String ip = request.getHeader("x-forwarded-for");
        if (ip == null || ip.isEmpty() || unknown.equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.isEmpty() || unknown.equalsIgnoreCase(ip)) {
            ip = request.getHeader("X-Forwarded-For");
        }
        if (ip == null || ip.isEmpty() || unknown.equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.isEmpty() || unknown.equalsIgnoreCase(ip)) {
            ip = request.getHeader("X-Real-IP");
        }
        if (ip == null || ip.isEmpty() || unknown.equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
            if (localhostIp.equalsIgnoreCase(ip) || localhostIpv6.equalsIgnoreCase(ip)) {
                // 根据网卡取本机配置的 IP
                InetAddress iNet = InetAddress.getLocalHost();
                if (iNet != null) {ip = iNet.getHostAddress();}
            }
        }
        // 对于通过多个代理的情况，分割出第一个 IP
        if (ip != null && ip.length() > ConstValue.MAX_IPV4_LEN) {
            if (ip.indexOf(separator) > 0) {
                ip = ip.substring(0, ip.indexOf(separator));
            }
        }
        return localhostIpv6.equals(ip) ? localhostIp : ip;
    }
}
