package com.cf.cs.crypto.arithmetic;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.Base64;

/**
 * The type Crypto helper.
 *
 * @author xajhuang @163.com
 */
@SuppressWarnings({"java:S5542", "java:S5547", "java:S4790"})
public class CryptoHelper {

    private static final String AES_ALGORITHM_STR = "AES/ECB/PKCS5Padding";
    private static final String DES_ALGORITHM_STR = "DES/ECB/PKCS5Padding";

    private static final String SHA1_PRNG = "SHA1PRNG";

    private CryptoHelper() {
        throw new AssertionError("Instantiating utility class.");
    }

    /**
     * Base 64 decryption byte [ ].
     *
     * @param ciphertext the ciphertext
     * @return the byte [ ]
     */
    public static byte[] base64Decryption(String ciphertext) {
        return Base64.getDecoder().decode(ciphertext);
    }

    /**
     * Base 64 encryption string.
     *
     * @param plaintext the plaintext
     * @return the string
     */
    public static String base64Encryption(byte[] plaintext) {
        return Base64.getEncoder().encodeToString(plaintext);
    }

    /**
     * Sha 256 encryption byte [ ].
     *
     * @param plaintext the plaintext
     * @return the byte [ ]
     * @throws NoSuchAlgorithmException the no such algorithm exception
     */
    public static byte[] sha256Encryption(String plaintext) throws NoSuchAlgorithmException {
        MessageDigest messageDigest = MessageDigest.getInstance("SHA-256");
        messageDigest.update(plaintext.getBytes(StandardCharsets.UTF_8));
        return messageDigest.digest();
    }

    /**
     * Md 5 encryption string.
     *
     * @param plaintext the plaintext
     * @return the string
     * @throws NoSuchAlgorithmException the no such algorithm exception
     */
    public static String md5Encryption(String plaintext) throws NoSuchAlgorithmException {
        MessageDigest messageDigest = MessageDigest.getInstance("MD5");
        messageDigest.update(plaintext.getBytes(StandardCharsets.UTF_8));
        return base64Encryption(messageDigest.digest());
    }

    /**
     * Aes 128 encryption byte [ ].
     *
     * @param plaintext the plaintext
     * @param aesKey    the aes key
     * @return the byte [ ]
     * @throws NoSuchAlgorithmException  the no such algorithm exception
     * @throws NoSuchPaddingException    the no such padding exception
     * @throws InvalidKeyException       the invalid key exception
     * @throws BadPaddingException       the bad padding exception
     * @throws IllegalBlockSizeException the illegal block size exception
     */
    public static byte[] aes128Encryption(byte[] plaintext,
                                          String aesKey) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException,
            BadPaddingException, IllegalBlockSizeException {
        KeyGenerator keyGen       = KeyGenerator.getInstance("AES");
        SecureRandom secureRandom = SecureRandom.getInstance(SHA1_PRNG);
        secureRandom.setSeed(aesKey.getBytes());

        keyGen.init(128, secureRandom);
        Cipher        cipher = Cipher.getInstance(AES_ALGORITHM_STR);
        SecretKeySpec key    = new SecretKeySpec(keyGen.generateKey().getEncoded(), "AES");
        cipher.init(Cipher.ENCRYPT_MODE, key);
        return cipher.doFinal(plaintext);
    }

    /**
     * Aes 128 decryption byte [ ].
     *
     * @param ciphertext the ciphertext
     * @param aesKey     the aes key
     * @return the byte [ ]
     * @throws NoSuchAlgorithmException  the no such algorithm exception
     * @throws NoSuchPaddingException    the no such padding exception
     * @throws InvalidKeyException       the invalid key exception
     * @throws BadPaddingException       the bad padding exception
     * @throws IllegalBlockSizeException the illegal block size exception
     */
    public static byte[] aes128Decryption(byte[] ciphertext,
                                          String aesKey) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException,
            BadPaddingException, IllegalBlockSizeException {
        KeyGenerator keyGen       = KeyGenerator.getInstance("AES");
        SecureRandom secureRandom = SecureRandom.getInstance(SHA1_PRNG);
        secureRandom.setSeed(aesKey.getBytes());

        keyGen.init(128, secureRandom);
        Cipher        cipher = Cipher.getInstance(AES_ALGORITHM_STR);
        SecretKeySpec key    = new SecretKeySpec(keyGen.generateKey().getEncoded(), "AES");
        cipher.init(Cipher.DECRYPT_MODE, key);
        return cipher.doFinal(ciphertext);
    }

    /**
     * Aes 256 encryption byte [ ].
     *
     * @param plaintext the plaintext
     * @param aesKey    the aes key
     * @return the byte [ ]
     * @throws NoSuchAlgorithmException  the no such algorithm exception
     * @throws NoSuchPaddingException    the no such padding exception
     * @throws InvalidKeyException       the invalid key exception
     * @throws BadPaddingException       the bad padding exception
     * @throws IllegalBlockSizeException the illegal block size exception
     */
    public static byte[] aes256Encryption(byte[] plaintext,
                                          String aesKey) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException,
            BadPaddingException, IllegalBlockSizeException {
        Cipher        cipher = Cipher.getInstance(AES_ALGORITHM_STR);
        SecretKeySpec key    = new SecretKeySpec(sha256Encryption(aesKey), "AES");
        cipher.init(Cipher.ENCRYPT_MODE, key);
        return cipher.doFinal(plaintext);
    }

    /**
     * Aes 256 decryption byte [ ].
     *
     * @param ciphertext the ciphertext
     * @param aesKey     the aes key
     * @return the byte [ ]
     * @throws NoSuchAlgorithmException  the no such algorithm exception
     * @throws NoSuchPaddingException    the no such padding exception
     * @throws InvalidKeyException       the invalid key exception
     * @throws BadPaddingException       the bad padding exception
     * @throws IllegalBlockSizeException the illegal block size exception
     */
    public static byte[] aes256Decryption(byte[] ciphertext,
                                          String aesKey) throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException,
            BadPaddingException, IllegalBlockSizeException {
        Cipher        cipher = Cipher.getInstance(AES_ALGORITHM_STR);
        SecretKeySpec key    = new SecretKeySpec(sha256Encryption(aesKey), "AES");
        cipher.init(Cipher.DECRYPT_MODE, key);
        return cipher.doFinal(ciphertext);
    }

    /**
     * Des decryption byte [ ].
     *
     * @param ciphertext the ciphertext
     * @param desKey     the des key
     * @return the byte [ ]
     * @throws InvalidKeyException       the invalid key exception
     * @throws NoSuchAlgorithmException  the no such algorithm exception
     * @throws NoSuchPaddingException    the no such padding exception
     * @throws BadPaddingException       the bad padding exception
     * @throws IllegalBlockSizeException the illegal block size exception
     */
    public static byte[] desDecryption(byte[] ciphertext,
                                       String desKey) throws InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException,
            BadPaddingException, IllegalBlockSizeException {
        KeyGenerator keyGen       = KeyGenerator.getInstance("DES");
        SecureRandom secureRandom = SecureRandom.getInstance(SHA1_PRNG);
        secureRandom.setSeed(desKey.getBytes());

        keyGen.init(56, secureRandom);
        Cipher        cipher = Cipher.getInstance(DES_ALGORITHM_STR);
        SecretKeySpec key    = new SecretKeySpec(keyGen.generateKey().getEncoded(), "DES");
        cipher.init(Cipher.DECRYPT_MODE, key);
        return cipher.doFinal(ciphertext);
    }

    /**
     * Deencryption byte [ ].
     *
     * @param plaintext the plaintext
     * @param desKey    the des key
     * @return the byte [ ]
     * @throws InvalidKeyException       the invalid key exception
     * @throws NoSuchAlgorithmException  the no such algorithm exception
     * @throws NoSuchPaddingException    the no such padding exception
     * @throws BadPaddingException       the bad padding exception
     * @throws IllegalBlockSizeException the illegal block size exception
     */
    public static byte[] desEncryption(byte[] plaintext,
                                       String desKey) throws InvalidKeyException, NoSuchAlgorithmException, NoSuchPaddingException,
            BadPaddingException, IllegalBlockSizeException {
        KeyGenerator keyGen       = KeyGenerator.getInstance("DES");
        SecureRandom secureRandom = SecureRandom.getInstance(SHA1_PRNG);
        secureRandom.setSeed(desKey.getBytes());

        keyGen.init(56, secureRandom);
        Cipher        cipher = Cipher.getInstance(DES_ALGORITHM_STR);
        SecretKeySpec key    = new SecretKeySpec(keyGen.generateKey().getEncoded(), "DES");
        cipher.init(Cipher.ENCRYPT_MODE, key);
        return cipher.doFinal(plaintext);
    }
}
