package com.cf.cs.database.service.impl;


import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.exception.CommonErrorCodeException;
import com.cf.cs.base.misc.ApiContextUtils;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.base.pojo.po.PageResults;
import com.cf.cs.database.mapper.ResourceDataMapper;
import com.cf.cs.database.mapper.RoleResourceMapper;
import com.cf.cs.database.mapper.UserMapper;
import com.cf.cs.database.pojo.entity.ResourceData;
import com.cf.cs.database.pojo.entity.RoleResource;
import com.cf.cs.database.pojo.entity.User;
import com.cf.cs.database.pojo.po.ResPermInfo;
import com.cf.cs.database.pojo.po.UserInfo;
import com.cf.cs.database.service.UserDataBaseService;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.spring.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

import static com.cf.cs.database.pojo.entity.table.RoleResourceTableDef.ROLE_RESOURCE;
import static com.cf.cs.database.pojo.entity.table.UserTableDef.USER;

/**
 * 用户表(User)表服务实现类
 *
 * @author makejava @163.com
 * @since 2023 -12-19 17:35:26
 */
@Service
@Slf4j
public class UserDataBaseServiceImpl extends ServiceImpl<UserMapper, User> implements UserDataBaseService {
    private static final String LOG_USER_NOTFOUND_MSG = "Current login user not found: [{}]";
    private static final String LOG_USER_NOT_EXISTS = "Current User is not exists.";

    @Resource
    private ResourceDataMapper resDataMapper;

    @Resource
    private RoleResourceMapper roleResourceMapper;

    @Resource
    private UserMapper userMapper;

    private User queryUser(QueryWrapper wrapper) {
        return this.getMapper().selectOneWithRelationsByQuery(wrapper);
    }

    @Override
    public User getAccountUserByName(String username) {
        QueryWrapper wrapper = QueryWrapper.create().from(USER).select().where(USER.USERNAME.eq(username));
        return (username == null || username.isEmpty()) ? null : queryUser(wrapper);
    }

    @Override
    public User getAccountUserByUid(String uid) {
        QueryWrapper wrapper = QueryWrapper.create().from(USER).select().where(USER.UID.eq(uid));
        return (!HelperUtils.stringNotEmptyOrNull(uid)) ? null : queryUser(wrapper);
    }

    @Override
    public UserInfo getUserInfoById(Long userId) {
        // 获取用户信息
        QueryWrapper wrapper = QueryWrapper.create().from(USER).select().where(USER.ID.eq(userId));
        User         user    = queryUser(wrapper);

        if (user == null) {
            log.error(LOG_USER_NOT_EXISTS);
            throw new CommonErrorCodeException(ErrorCode.ERR_USERNOTFOUND);
        }

        // 低优先级用户不能获取高优先级用户信息
        if (verifyCurrentExceedPermission(user)) {
            throw new CommonErrorCodeException(ErrorCode.ERR_PERMISSION);
        }

        return UserInfo.builder()
                       .uid(user.getUid())
                       .createdTime(user.getCreatedTime())
                       .gender(user.getGender())
                       .username(user.getUsername())
                       .roleId(user.getRoleId())
                       .userStatus(user.getUserCredential().getState())
                       .build();
    }

    @Override
    public UserInfo getUserInfoByUid(String uid) {
        // 获取用户信息
        QueryWrapper wrapper = QueryWrapper.create().from(USER).select().where(USER.UID.eq(uid));
        User         user    = queryUser(wrapper);

        if (user == null) {
            log.error(LOG_USER_NOT_EXISTS);
            throw new CommonErrorCodeException(ErrorCode.ERR_USERNOTFOUND);
        }

        // 低优先级用户不能获取高优先级用户信息
        if (verifyCurrentExceedPermission(user)) {
            throw new CommonErrorCodeException(ErrorCode.ERR_PERMISSION);
        }

        return UserInfo.builder()
                       .uid(user.getUid())
                       .createdTime(user.getCreatedTime())
                       .gender(user.getGender())
                       .username(user.getUsername())
                       .roleId(user.getRoleId())
                       .userStatus(user.getUserCredential().getState())
                       .build();
    }

    @Override
    public UserInfo getUserInfoByUsername(String username) {
        // 获取用户信息
        QueryWrapper wrapper = QueryWrapper.create().from(USER).select().where(USER.USERNAME.eq(username));
        User         user    = queryUser(wrapper);

        if (user == null) {
            log.error("Current User({}) is not exists.", username);
            throw new CommonErrorCodeException(ErrorCode.ERR_USERNOTFOUND);
        }

        return getUserInfoById(user.getId());
    }

    @Override
    public UserInfo getCurrentUserInfo() {
        String username = ApiContextUtils.get().getUsername();
        User   user     = getAccountUserByName(username);

        if (user == null) {
            log.error(LOG_USER_NOTFOUND_MSG, username);
            throw new CommonErrorCodeException(ErrorCode.ERR_TOKENNOTFOUND);
        }

        return UserInfo.builder()
                       .uid(user.getUid())
                       .createdTime(user.getCreatedTime())
                       .gender(user.getGender())
                       .username(user.getUsername())
                       .roleId(user.getRoleId())
                       .userStatus(user.getUserCredential().getState())
                       .build();
    }

    @Override
    public PageResults<UserInfo> getAllUserInfo(Long pageNumber, Long pageSize, Long totalSize) {
        QueryWrapper   wrapper = QueryWrapper.create().from(USER).select();
        List<UserInfo> ui      = new ArrayList<>();

        Page<User> user = userMapper.paginateWithRelations(pageNumber, pageSize, totalSize, wrapper);

        user.getRecords()
            .forEach(k -> ui.add(UserInfo.builder()
                                         .uid(k.getUid())
                                         .createdTime(k.getCreatedTime())
                                         .gender(k.getGender())
                                         .username(k.getUsername())
                                         .roleId(k.getRoleId())
                                         .userStatus(k.getUserCredential().getState())
                                         .build()));

        return PageResults.<UserInfo>builder()
                          .items(ui)
                          .pageNumber(user.getPageNumber())
                          .pageSize(user.getPageSize())
                          .totalPage(user.getTotalPage())
                          .totalRow(user.getTotalRow())
                          .build();
    }

    @Override
    public List<ResPermInfo> getCurrentUserResourcePerm() {
        List<ResourceData> res;

        String username = ApiContextUtils.get().getUsername();
        User   user     = getAccountUserByName(username);

        if (user == null) {
            log.error(LOG_USER_NOTFOUND_MSG, username);
            throw new CommonErrorCodeException(ErrorCode.ERR_TOKENNOTFOUND);
        }

        List<ResPermInfo>  result  = new ArrayList<>();
        QueryWrapper       wrapper = QueryWrapper.create().from(ROLE_RESOURCE).select().where(ROLE_RESOURCE.ROLE_ID.eq(user.getId()));
        List<RoleResource> roleRes = roleResourceMapper.selectListWithRelationsByQuery(wrapper);

        roleRes.forEach(k -> result.add(
                ResPermInfo.builder()
                           .id(k.getResourceId())
                           .name(k.getResources().getName())
                           .path(k.getResources().getPath())
                           //.resType(k.getResources().getResType().getTypeId())
                           .resTypeId(k.getResources().getResType().getTypeId())
                           .httpMethod(k.getResources().getHttpMethod())
                           .enable(k.getAuthorize())
                           .build()));

        res = resDataMapper.selectAllWithRelations()
                           .stream()
                           .filter(k -> roleRes.stream().noneMatch(m -> m.getResources().getPath().equals(k.getPath())))
                           .toList();

        res.forEach(k -> result.add(ResPermInfo.builder()
                                               .id(k.getId())
                                               .name(k.getName())
                                               .path(k.getPath())
                                               .httpMethod(k.getHttpMethod())
                                               .resTypeId(k.getResType().getTypeId())
                                               .enable(false)
                                               .build()));

        return result;
    }

    @Override
    public List<ResPermInfo> getUserResourcePerm(Long userId) {
        List<ResourceData> res;

        // 获取用户信息
        QueryWrapper wrapper = QueryWrapper.create().from(USER).select().where(USER.ID.eq(userId));
        User         user    = queryUser(wrapper);

        if (user == null) {
            log.error(LOG_USER_NOT_EXISTS);
            throw new CommonErrorCodeException(ErrorCode.ERR_USERNOTFOUND);
        }

        // 低优先级用户不能获取高优先级用户信息
        if (verifyCurrentExceedPermission(user)) {
            throw new CommonErrorCodeException(ErrorCode.ERR_PERMISSION);
        }

        List<ResPermInfo> result = new ArrayList<>();
        wrapper = QueryWrapper.create().from(ROLE_RESOURCE).select().where(ROLE_RESOURCE.ROLE_ID.eq(user.getId()));
        List<RoleResource> roleRes = roleResourceMapper.selectListWithRelationsByQuery(wrapper);

        roleRes.forEach(k -> result.add(ResPermInfo.builder()
                                                   .id(k.getResourceId())
                                                   .name(k.getResources().getName())
                                                   .path(k.getResources().getPath())
                                                   .httpMethod(k.getResources().getHttpMethod())
                                                   //.resType(k.getResources().getResType().getTypeId())
                                                   .resTypeId(k.getResources().getResType().getTypeId())
                                                   .enable(k.getAuthorize())
                                                   .build()));

        res = resDataMapper.selectAllWithRelations()
                           .stream()
                           .filter(k -> roleRes.stream().noneMatch(m -> m.getResources().getPath().equals(k.getPath())))
                           .toList();

        res.forEach(k -> result.add(ResPermInfo.builder()
                                               .id(k.getId())
                                               .name(k.getName())
                                               .path(k.getPath())
                                               .resTypeId(k.getResType().getTypeId())
                                               .httpMethod(k.getHttpMethod())
                                               .enable(false)
                                               .build()));

        return result;
    }

    private boolean verifyCurrentExceedPermission(User accessUser) {
        String loginUsername = ApiContextUtils.get().getUsername();

        if (!HelperUtils.stringNotEmptyOrNull(loginUsername)) {
            log.error(LOG_USER_NOTFOUND_MSG, accessUser.getUsername());
            throw new CommonErrorCodeException(ErrorCode.ERR_TOKENNOTFOUND);
        }

        User logUser = getAccountUserByName(loginUsername);

        if (logUser == null) {
            log.error(LOG_USER_NOTFOUND_MSG, loginUsername);
            throw new CommonErrorCodeException(ErrorCode.ERR_TOKENNOTFOUND);
        }

        // 低优先级用户不能获取高优先级用户信息
        if (logUser.getId() <= accessUser.getRoleId()) {
            return false;
        }

        log.error("Permission check deny: login user{}, request user{}", logUser.getId(), accessUser.getRoleId());
        return true;
    }
}
