package com.cf.cs.integratetest.common;


import com.cf.cs.base.common.ConstValue;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.common.ProtoCryptoType;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.integratetest.CsIntegrateTestApplication;
import com.cf.cs.protocol.config.ProtocolConfigure;
import com.cf.cs.protocol.misc.ProtocolJsonUtils;
import com.cf.cs.protocol.pojo.dto.LoginReq;
import com.cf.cs.protocol.pojo.dto.ProtocolReq;
import com.cf.cs.protocol.pojo.po.BaseRespStatus;
import com.cf.cs.protocol.pojo.vo.LoginResp;
import com.cf.cs.protocol.pojo.vo.ProtocolResp;
import com.cf.cs.protocol.service.ProtocolSecurityService;
import jakarta.annotation.Resource;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.TestInstance;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.request.MockHttpServletRequestBuilder;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.web.bind.annotation.RequestMethod;

import java.util.Objects;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.DEFINED_PORT, classes = {CsIntegrateTestApplication.class})
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
@ActiveProfiles({"user", "test"})
public abstract class TestBaseAuthentication {
    protected static final ThreadLocal<HttpHeaders> headersThreadLocal = new ThreadLocal<>();

    @Autowired
    protected MockMvc mockMvc;

    @Resource
    private ProtocolSecurityService securityService;

    public HttpHeaders getHeaders() {
        return headersThreadLocal.get();
    }

    @BeforeAll
    public void setUpClass() throws Exception {
        LoginReq logReq = LoginReq.builder()
                                  .username("admin")
                                  .password("8c6976e5b5410415bde908bd4dee15dfb167a9c873fc4bb8a81f6f2ab448a918")
                                  .build();

        ProtocolReq<LoginReq> loginPro = new ProtocolReq<>();
        loginPro.setMsgContent(logReq);
        loginPro.setVer(1);
        loginPro.setTimeStamp(System.currentTimeMillis());
        loginPro.setCryptoType(ProtocolConfigure.SECURITY_PROTOCOL_TYPE);

        MvcResult mvcResult = mockMvc.perform(post("/api/login")
                                                      .contentType(MediaType.APPLICATION_JSON) // Set the content type to JSON
                                                      .content(HelperUtils.getJson(loginPro))) // Pass the JSON request body
                                     .andExpect(status().isOk()) // Expect that the status is 200 OK
                                     .andReturn();

        // 假设JWT在响应体中返回。您需要根据实际情况调整此行代码。
        String respJson = mvcResult.getResponse().getContentAsString();
        Assertions.assertNotNull(respJson);
        Assertions.assertNotEquals(0, respJson.length());
        ProtocolResp<?> resp = ProtocolJsonUtils.jsonGetProtocolResp(respJson, LoginResp.class);

        Assertions.assertNotNull(resp);
        Assertions.assertNotNull(resp.getMsgContent());
        Assertions.assertEquals(resp.getCode(), HttpStatus.OK.value());

        LoginResp loginResp = (LoginResp) resp.getMsgContent();

        Assertions.assertEquals(loginResp.getStatus(), ErrorCode.ERR_OK.getCode());
        Assertions.assertNotNull(loginResp.getToken());
        Assertions.assertNotEquals(0, loginResp.getToken().length());

        HttpHeaders headers = new HttpHeaders();
        MediaType   type    = MediaType.parseMediaType("application/json; charset=UTF-8");
        headers.setContentType(type);
        headers.add("Accept", MediaType.APPLICATION_JSON.toString());
        headers.add(HttpHeaders.ACCEPT_ENCODING, "gzip, deflate, br");
        headers.setBearerAuth(loginResp.getToken());
        headersThreadLocal.set(headers);
    }

    public <T> Object performanceRestful(RequestMethod reqType, T reqObject, String urlPath, Class<?>[] subRespClass) throws Exception {
        MockHttpServletRequestBuilder build = createMockMvcBuilder(urlPath, reqType);

        if (Objects.nonNull(reqObject)) {
            String         sendMsgContent;
            ProtocolReq<T> reqInfo = new ProtocolReq<>();
            reqInfo.setVer(ConstValue.Protocol.VERSION);
            reqInfo.setCryptoType(ProtocolConfigure.SECURITY_PROTOCOL_TYPE);
            reqInfo.setTimeStamp(System.currentTimeMillis());
            reqInfo.setMsgContent(reqObject);

            if (ProtocolConfigure.SECURITY_PROTOCOL_TYPE != ProtoCryptoType.CRYPTO_NONE.getValue()) {
                String cipherText = securityService.encryptProtocolString(HelperUtils.getJson(reqObject),
                                                                          ProtocolConfigure.SECURITY_PROTOCOL_TYPE);

                ProtocolReq<String> cipherInfo = new ProtocolReq<>();
                cipherInfo.setVer(reqInfo.getVer());
                cipherInfo.setCryptoType(reqInfo.getCryptoType());
                cipherInfo.setTimeStamp(System.currentTimeMillis());
                cipherInfo.setMsgContent(cipherText);
                sendMsgContent = HelperUtils.getJson(cipherInfo);
            } else {
                sendMsgContent = HelperUtils.getJson(reqInfo);
            }

            build.content(sendMsgContent);
        }


        String rspValue = mockMvc.perform(build)
                                 .andDo(print())
                                 .andReturn()
                                 .getResponse()
                                 .getContentAsString();

        AssertValidString(rspValue);

        rspValue = securityService.decryptProtocol(rspValue);
        AssertValidString(rspValue);

        ProtocolResp<?> resp = ProtocolJsonUtils.jsonGetProtocolResp(rspValue, subRespClass);

        Assertions.assertNotNull(resp);
        Assertions.assertNotNull(resp.getMsgContent());
        Assertions.assertEquals(resp.getCode(), HttpStatus.OK.value());

        return resp.getMsgContent();
    }

    private MockHttpServletRequestBuilder createMockMvcBuilder(String uri, RequestMethod reqType) {
        MockHttpServletRequestBuilder build = switch (reqType) {
            case PUT -> MockMvcRequestBuilders.put(uri);
            case GET -> MockMvcRequestBuilders.get(uri);
            case DELETE -> MockMvcRequestBuilders.delete(uri);
            default -> MockMvcRequestBuilders.post(uri);
        };

        build.accept(MediaType.APPLICATION_JSON);
        build.headers(getHeaders());
        return build;
    }

    public void AssertValidString(String str) {
        Assertions.assertNotNull(str);
        Assertions.assertFalse(str.isEmpty());
    }

    public <T extends BaseRespStatus> void AssertValidCommonResp(T resp) {
        Assertions.assertNotNull(resp);
        Assertions.assertNotNull(resp.getStatus());
        Assertions.assertNotNull(resp.getMessage());
        Assertions.assertNotEquals(0, resp.getMessage().length);
        Assertions.assertEquals(resp.getStatus(), ErrorCode.ERR_OK.getCode());
    }
}
